<?php

namespace CRM\Factory;

use Doctrine\ORM\EntityManagerInterface;
use CRM\Traits\TQuery;

class TaskScheduleFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManagerInterface  $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * Retrieves task schedule data based on lead information.
     *
     * @param array $paramsQuery The query parameters.
     * @return array The task schedule data.
     * @throws \Exception If an error occurs.
     */
    public function getTaskScheduleBasedOnLead(array $paramsQuery): array
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();

            $queryBuilder
                ->select([
                    "FILACONTATO.NRSEQFILACONTA",
                    "FILACONTATO.DSFILACONTATO",
                    "FILACONTATO.DTPLANFILACONTA AS DTHRPLANFILACONTA",
                    "FILACONTATO.DTREALFILACONTA",
                    "FILACONTATO.NRPRIORFILACONT",
                    "FILACONTATO.CDOPERREAL",
                    "FILACONTATO.NRPARCNEGOCIO",
                    "FILACONTATO.NRRELACIONAPARC",
                    "FILACONTATO.DSTELEFONETAREFA",
                    "FILACONTATO.DSEMAILTAREFA",
                    "PARCNEGOCIO.NMPRINCIPALPARC",
                    "FILACONTATO.NRSEQCLASSIFICATAREFA",
                    "FILACONTATO.DSDETEFILACONTA",
                    "FILACONTATO.IDORIGFILACONTA",
                    "OPORTUNIDADE.NRSEQCONTATO AS TIPO",
                    "FILACONTATO.CDFILIAL",
                    "FILACONTATO.NRORG",
                    "FILACONTATO.CRM_NRORGTRAB",
                    "FILACONTATO.IDATIVO",
                    "FILACONTATO.NRSEQINTERLOCUTOR",
                    "INTERLOCUTOR.NMINTERLOCUTOR",
                    "CLASSIFICATAREFA.DSCLASSIFICATAREFA",
                    "FILACONTATO.NRCONTATO",
                    "OPERADOR.NMOPERADOR",
                    "FILACONTATO.DTVENCFILACONTA",
                    "CONTATO.DSREFECONT AS DESCRICAO"
                ])
                ->from('FILACONTATO', 'FILACONTATO')
                ->leftJoin('FILACONTATO', 'CRM_OPORTUNI', 'OPORTUNIDADE',  'FILACONTATO.NRSEQCONTATO = OPORTUNIDADE.NRSEQCONTATO AND FILACONTATO.NRORG = OPORTUNIDADE.NRORG')
                ->leftJoin('FILACONTATO', 'CONTATO', 'CONTATO', 'FILACONTATO.NRSEQCONTATO = CONTATO.NRSEQCONTATO AND FILACONTATO.NRORG = CONTATO.NRORG')
                ->leftJoin('FILACONTATO', 'OPERADOR', 'OPERADOR', 'FILACONTATO.CDOPERREAL = OPERADOR.CDOPERADOR AND FILACONTATO.NRORG = OPERADOR.NRORG')
                ->leftJoin('FILACONTATO', 'CRM_CLASSIFICATAREFA', 'CLASSIFICATAREFA', 'FILACONTATO.NRSEQCLASSIFICATAREFA = CLASSIFICATAREFA.NRSEQCLASSIFICATAREFA AND FILACONTATO.IDATIVO = CLASSIFICATAREFA.IDATIVO AND FILACONTATO.NRORG = CONTATO.NRORG')
                ->leftJoin('FILACONTATO', 'PARCNEGOCIO', 'PARCNEGOCIO', 'FILACONTATO.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO AND FILACONTATO.NRORG = PARCNEGOCIO.NRORG')
                ->leftJoin('FILACONTATO', 'CRM_INTERLOCUTOR', 'INTERLOCUTOR', 'FILACONTATO.NRSEQINTERLOCUTOR = INTERLOCUTOR.NRSEQINTERLOCUTOR AND FILACONTATO.NRORG = INTERLOCUTOR.NRORG')
                ->where('FILACONTATO.NRORG = :NRORG')
                ->andWhere('FILACONTATO.IDATIVO = :IDATIVO')
                ->andWhere('FILACONTATO.NRSEQCONTATO = :NRSEQCONTATO')
                ->orderBy('FILACONTATO.DTPLANFILACONTA', 'DESC');

            $queryBuilder->setParameters($paramsQuery);
            return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retrieves task classification data based on query parameters.
     *
     * @param array $paramsQuery The query parameters.
     * @return array The task classification data.
     * @throws \Exception If an error occurs.
     */
    public function getTaskClassification(array $paramsQuery): array
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();

            $queryBuilder
                ->select([
                    "CRM_CLASSIFICATAREFA.NRORG",
                    "CRM_CLASSIFICATAREFA.IDATIVO",
                    "CRM_CLASSIFICATAREFA.DTULTATU",
                    "CRM_CLASSIFICATAREFA.DTINCLUSAO",
                    "CRM_CLASSIFICATAREFA.NRORGULTATU",
                    "CRM_CLASSIFICATAREFA.CDOPERULTATU",
                    "CRM_CLASSIFICATAREFA.NRORGINCLUSAO",
                    "CRM_CLASSIFICATAREFA.CDOPERINCLUSAO",
                    "CRM_CLASSIFICATAREFA.DSCLASSIFICATAREFA",
                    "CRM_CLASSIFICATAREFA.NRSEQCLASSIFICATAREFA",
                ])
                ->from('CRM_CLASSIFICATAREFA', 'CRM_CLASSIFICATAREFA')
                ->where('CRM_CLASSIFICATAREFA.NRORG = :NRORG')
                ->andWhere('CRM_CLASSIFICATAREFA.IDATIVO = :IDATIVO');

            $queryBuilder->setParameters($paramsQuery);
            return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Checks if there are any open tasks based on the provided query parameters.
     *
     * @param array $paramsQuery An array containing query parameters.
     * @return array An array containing information about open tasks.
     * @throws \Exception if an error occurs.
     */
    public function checkIfThereAreAnyOpenTasks(array $paramsQuery): array
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();

            $queryBuilder
                ->select([
                    "FC.NRSEQFILACONTA",
                    "FC.CDOPERREAL",
                    "OP.NMOPERADOR",
                    "FC.DTPLANFILACONTA"
                ])
                ->from('FILACONTATO', 'FC')
                ->leftJoin('FC', 'PARCNEGOCIO', 'PN', 'FC.NRPARCNEGOCIO = PN.NRPARCNEGOCIO AND FC.NRORG = PN.NRORG')
                ->leftJoin('FC', 'OPERADOR', 'OP', 'FC.CDOPERREAL = OP.CDOPERADOR AND FC.NRORG = OP.NRORG')
                ->where('FC.NRPARCNEGOCIO = :NRPARCNEGOCIO')
                ->andWhere('FC.NRORG = :NRORG')
                ->andWhere('FC.CRM_NRORGTRAB = :CRM_NRORGTRAB')
                ->andWhere('DTREALFILACONTA IS NULL')
                ->andWhere('FC.IDATIVO = :IDATIVO')
                ->andWhere('PN.IDATIVO = :IDATIVO');

            $queryBuilder->setParameters($paramsQuery);
            return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Inserts a new lead evolution and schedules the next task based on the provided query parameters.
     *
     * @param array $paramsQuery An array containing parameters for inserting the lead evolution.
     * @throws \Exception if an error occurs.
     */
    public function insertLeadEvolNextTask(array $paramsQuery): void
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();

            $queryBuilder->insert('FILACONTATO')
                ->values([
                    'DSFILACONTATO' => ':DSFILACONTATO',
                    'DTPLANFILACONTA' => ':DTPLANFILACONTA',
                    'CDOPERREAL' => ':CDOPERREAL',
                    'NRSEQFILACONTA' => ':NRSEQFILACONTA',
                    'NRORG' => ':NRORG',
                    'CRM_NRORGTRAB' => ':CRM_NRORGTRAB',
                    'NRORGINCLUSAO' => ':NRORGINCLUSAO',
                    'CDOPERINCLUSAO' => ':CDOPERINCLUSAO',
                    'NRPARCNEGOCIO' => ':NRPARCNEGOCIO',
                    'IDATIVO' => ':IDATIVO',
                    'NRCONTATO' => ':NRCONTATO',
                    'NRPRIORFILACONT' => ':NRPRIORFILACONT',
                    'NRSEQCONTATO' => ':NRSEQCONTATO',
                    'NRSEQCLASSIFICATAREFA' => ':NRSEQCLASSIFICATAREFA',
                    'NRSEQINTERLOCUTOR' => ':NRSEQINTERLOCUTOR',
                    'DSEMAILTAREFA' => ':DSEMAILTAREFA',
                    'DSTELEFONETAREFA' => ':DSTELEFONETAREFA'
                ]);

            $queryBuilder->setParameters($paramsQuery);

            $this->connection->executeStatement($queryBuilder->getSQL(), $queryBuilder->getParameters());
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
