<?php

namespace CRM\Service;

use CRM\Util\Utilities;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use CRM\Factory\EvolucaoFactory;
use Teknisa\Libs\Util\Environment;

class EvolucaoService
{
    protected $newCode;
    protected $connection;
    protected $Environment;
    protected $EvolucaoFactory;

    public function __construct(Environment $Environment, NewCode $newCode, EntityManager $entityManager, EvolucaoFactory $EvolucaoFactory)
    {
        $this->EvolucaoFactory                    = $EvolucaoFactory;
        $this->Environment                        = $Environment;
        $this->newCode                            = $newCode;
        $this->connection                         = $entityManager->getConnection();
    }

    /**
     * Creates a new evolution control number based on the provided table name and organization number.
     *
     * @param string $tableName The name of the table for which the code is generated.
     * @param int $nrorg The organization number.
     * @return string The newly generated evolution control number.
     * @throws \Throwable if an error occurs during code generation.
     */
    private function createNewNRSEQEVOLCONT($tableName, $nrorg): string
    {
        try {
            return $this->newCode->getCodeByTableName($tableName, $nrorg);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    /**
     * Get evolution tab data based on provided parameters.
     *
     * @param array $params An array of parameters containing NRCONTATO and NRSEQCONTATO.
     * @return array The evolution tab data.
     * @throws \Exception If an error occurs while retrieving the data.
     */
    public function getEvolutionTab(array $params): array
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();

            // Prepare the query parameters
            $paramsQuery = [
                'NRORG' => $nrorg,
                'IDATIVO' => 'S',
                'CRM_NRORGTRAB' => $nrorg,
                'NRCONTATO' => $params['NRCONTATO'], // Ensure that NRCONTATO is provided
                'NRSEQCONTATO' => $params['NRSEQCONTATO'], // Ensure that NRSEQCONTATO is provided
            ];

            // Retrieve evolution tab data using the factory
            return $this->EvolucaoFactory->getEvolutionTab($paramsQuery);
        } catch (\Exception $e) {
            // Handle the exception appropriately, such as logging or reporting the error
            throw $e; // Or do something more specific here, like throwing a custom exception
        }
    }

    /**
     * Retrieves contact types based on organization number.
     *
     * @return array An array containing contact types.
     * @throws \Exception if an error occurs.
     */
    public function getContactType(): array
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();

            // Prepare the query parameters
            $paramsQuery = [
                'NRORG' => $nrorg,
                'IDATIVO' => 'S',
                'CRM_NRORGTRAB' => $nrorg
            ];

            return $this->EvolucaoFactory->getContactType($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retrieves evolution reasons based on organization number and current user id.
     *
     * @return array An array containing evolution reasons.
     * @throws \Exception if an error occurs.
     */
    public function getEvolutionReason(): array
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();
            // Get Current User Id
            $cdOperador = $this->Environment->getCdOperador();

            // Prepare the query parameters
            $paramsQuery = [
                'NRORG' => $nrorg,
                'IDATIVO' => 'S',
                'CDOPERADOR' => $cdOperador,
            ];

            return $this->EvolucaoFactory->getEvolutionReason($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retrieves evolution reasons based on organization number and current user id.
     *
     * @return array An array containing evolution reasons based on the current operator.
     * @throws \Exception if an error occurs.
     */
    public function getEvolutionReasonBasedOnOperator(): array
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();
            // Get Current User Id
            $cdOperador = $this->Environment->getCdOperador();

            // Prepare the query parameters
            $paramsQuery = [
                'NRORG' => $nrorg,
                'IDATIVO' => 'S',
                'CDOPERADOR' => $cdOperador,
            ];

            return $this->EvolucaoFactory->getEvolutionReasonBasedOnOperator($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retrieves results based on organization number.
     *
     * @return array An array containing results.
     * @throws \Exception if an error occurs.
     */
    public function getResult(): array
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();

            // Prepare the query parameters
            $paramsQuery = [
                'NRORG' => $nrorg,
                'IDATIVO' => 'S',
            ];

            return $this->EvolucaoFactory->getResult($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retrieves attachments based on organization number, contact number, and contact sequence number.
     *
     * @param array $row An array containing contact information.
     * @return array An array containing attachments.
     * @throws \Exception if an error occurs.
     */
    public function getAttachment($row): array
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();

            // Prepare the query parameters
            $paramsQuery = [
                'NRORG' => $nrorg,
                'IDATIVO' => 'S',
                'CRM_NRORGTRAB' => $nrorg,
                'NRCONTATO' => $row['NRCONTATO'], // Ensure that NRCONTATO is provided
                'NRSEQCONTATO' => $row['NRSEQCONTATO'], // Ensure that NRSEQCONTATO is provided
            ];

            return $this->EvolucaoFactory->getAttachment($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Saves a new evolution based on provided data.
     *
     * @param array $row An array containing evolution data.
     * @return string The identifier of the newly saved evolution.
     * @throws \Exception if an error occurs.
     */
    public function saveNewEvolution($row): string
    {
        try {
            // Get the organization number
            $nrorg = $this->Environment->getNrOrg();
            $cdfilial = $this->Environment->getCdFilial();
            $nrseqevolcont = $this->createNewNRSEQEVOLCONT('EVOLCONT', $nrorg);

            $paramsQuery = [
                'IDATIVO'               => 'S',
                'NRORG'                 => $nrorg,
                'CRM_NRORGTRAB'         => $nrorg,
                'NRORGINCLUSAO'         => $nrorg,
                'CDFILIAL'              => $cdfilial,
                'NRSEQEVOLCONT'         => $nrseqevolcont,
                'DTHREVOLCONT'          => $row['DTHREVOLCONT'],
                'NRCONTATO'             => $row['NRCONTATO'],
                'CDOPEREVOL'            => $row['CDOPEREVOL'],
                'TXEVOLCONT'            => $row['TXEVOLCONT'],
                'DSEVOLCONT'            => $row['TXEVOLCONT'],
                'CDTIPOCONT'            => $row['CDTIPOCONT'],
                'CDOPERINCLUSAO'        => $row['CDOPEREVOL'],
                'NRSEQCONTATO'          => $row['NRSEQCONTATO'],
                'IDSENEVOCONT'          => $row['IDSENEVOCONT'],
                'IDGERAFATURA'          => $row['IDGERAFATURA'],
                'IDTPEVOLCONT'          => $row['IDTPEVOLCONT'],
                'CDMOTIEVOCON'          => $row['CDMOTIEVOCON'],
                'NRSEQRESULEVOL'        => $row['NRSEQRESULEVOL'],
                'IDANEXOEVOLCONT'       => $row['IDANEXOEVOLCONT'],
                'NRRELACIONAPARC'       => $row['NRRELACIONAPARC'],
                'NRSEQINTERLOCUTOR'     => $row['NRSEQINTERLOCUTOR'],
                'NMINTEVOCONT'          => $row['NMINTEVOCONT'] ?? null,
                'NRSEQFILACONTA'        => $row['NRSEQFILACONTATO'] ?? null,
            ];

            return $this->EvolucaoFactory->saveNewEvolution($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
